# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gbf_beautify_honors']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.0.3,<9.0.0',
 'dataclasses-json>=0.5.7,<0.6.0',
 'ortools>=9.3.10497,<10.0.0',
 'tabulate>=0.8.9,<0.9.0',
 'types-setuptools>=57.4.14,<58.0.0',
 'types-tabulate>=0.8.8,<0.9.0',
 'wcwidth>=0.2.5,<0.3.0']

entry_points = \
{'console_scripts': ['gbf_beautify_honors = gbf_beautify_honors.main:main']}

setup_kwargs = {
    'name': 'gbf-beautify-honors',
    'version': '0.1.0',
    'description': 'A CLI tool to help you beautify Guild War honors',
    'long_description': '# Granblue Fantasay - Beautify Honors\nA CLI tool to help you figure out how to beautify honors in the Guild War event. (古戦場の貢献度調整)\n\nRead this in other languages: [English](README.md), [中文](README.zh-tw.md).\n\n\n![sample_result](assets/sample_result.png)\n\n## Prerequisites\nPlease read at least one of these well-written tutorials to known how to get the exact honors.\n- https://kamigame.jp/%E3%82%B0%E3%83%A9%E3%83%96%E3%83%AB/%E3%82%A4%E3%83%99%E3%83%B3%E3%83%88/%E6%B1%BA%E6%88%A6%EF%BC%81%E6%98%9F%E3%81%AE%E5%8F%A4%E6%88%A6%E5%A0%B4/%E8%B2%A2%E7%8C%AE%E5%BA%A6%E8%AA%BF%E6%95%B4.html\n- https://gbf.wiki/User:Midokuni/Notepad/Beauty_of_Honor\n- https://forum.gamer.com.tw/C.php?bsn=25204&snA=11313\n\n\n## System requirements\n- Python 3.7+\n\n\n## Install requirements\nYou can use `pip` or `poetry` to install requirements.\n\n### pip\n```sh\n$ pip install -r requirements.txt\n```\n\n### poetry\n```sh\n$ poetry install\n```\n\n## How to use\n\n1. Solo the NM Bosses until the difference between your current honors and expected honors is roughly greater than one million. An appropriate gap is a good start because there may be a greater chance of finding a good way to achieve the goal.\n2. Run the cli tool in interactive mode or direct mode. Help page is simply shown below:\n\n```sh\n$ python gbf_beautify_honors/main.py --help\nUsage: main.py [OPTIONS]\n\nOptions:\n  --current INTEGER   Your current honors  [required]\n  --expected INTEGER  Your expected honors  [required]\n  --config PATH       Custom config path\n  --help              Show this message and exit.\n```\n\n### Interactive mode example\n```python\n$ python gbf_beautify_honors/main.py\nYour current honors : 1398542611\nYour expected honors: 1400000000\nCustom config path []:\n```\n\n### Direct mode example\n```python\n$ python gbf_beautify_honors/main.py --current=1398542611 --expected=1400000000\n```\n\n\n## Examples\nNext, we will use some examples to explain how to use this tool, and how to adjust the configuration file.\n\n### Case 1: There is a solution\n```python\n$ python gbf_beautify_honors/main.py\nYour current honors : 1398542611\nYour expected honors: 1400000000\nCustom config path []:\n╒═══════════════════════════════════════╤═════════╤═════════════════╕\n│ Action                                │   Honor │   Optimal Times │\n╞═══════════════════════════════════════╪═════════╪═════════════════╡\n│ Eyeball VH (0 button)                 │    8000 │               1 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Meat Beast VH (0 button)              │   21400 │               4 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Meat Beast EX (0 button)              │   50578 │               3 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Meat Beast EX+ (0 button)             │   80800 │              10 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Meat Beast EX+ (1 summon)             │   80810 │               5 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Join raid and only use Break Assassin │       1 │               5 │\n╘═══════════════════════════════════════╧═════════╧═════════════════╛\n```\n\n### Case 2: There is no solution\nBasically, there is always a solution because we can join raid and only use Break Assassin to get exactly 1 honor.\nHowever, this is usually an unrealistic approach, so the default config makes some constraints on the maximum time on each type of battle. This leads to the fact that sometimes it is not possible to find a solution.\n```\n$ python gbf_beautify_honors/main.py\nYour current honors : 1399999900\nYour expected honors: 1400000000\nCustom config path []:\nThere is no solution to achieve the expected honors. Please relax the constraints and try again.\n```\n\nTo solve this problem, we can use custom config to relax the constraints to find a solution.\n\n1. Download the example [config.json](example_configs/config.json).\n2. Modify the `max_accepatable_times` of the action "Join raid and only use Break Assassin" to `100`.\n3. Re-run the script with custom config.\n```sh\n$ python gbf_beautify_honors/main.py\nYour current honors : 1399999900\nYour expected honors: 1400000000\nCustom config path []: config.json\n╒═══════════════════════════════════════╤═════════╤═════════════════╕\n│ Action                                │   Honor │   Optimal Times │\n╞═══════════════════════════════════════╪═════════╪═════════════════╡\n│ Join raid and only use Break Assassin │       1 │             100 │\n╘═══════════════════════════════════════╧═════════╧═════════════════╛\n```\n\nThe configuration is flexible and you can try to modify different values in it, re-run the script and see if there is an solution. You can also add self-defined action into the config.json as long as you know you can get the exact honor value from this action, e.g. you can add this object into the actions list,\n```\n{\n    "name": "Some custom action for demo",\n    "honor": 11,\n    "max_accepatable_times": 10\n}\n```\n\nRe-run the script, it just works!\n```\n$ python gbf_beautify_honors/main.py\nYour current honors : 1399999900\nYour expected honors: 1400000000\nCustom config path []: config.json\n╒═══════════════════════════════════════╤═════════╤═════════════════╕\n│ Action                                │   Honor │   Optimal Times │\n╞═══════════════════════════════════════╪═════════╪═════════════════╡\n│ Join raid and only use Break Assassin │       1 │               1 │\n├───────────────────────────────────────┼─────────┼─────────────────┤\n│ Some custom action for demo           │      11 │               9 │\n╘═══════════════════════════════════════╧═════════╧═════════════════╛\n```\n\n\n\n## How it works\nWe can formulate this problem as an integer programming problem and solve it using the [OR-Tools](https://developers.google.com/optimization)\n\nIs this case, we use an integer variable h<sub>i</sub> to represent the exact honor earned from battle `i` (`i` can be `Eyeball N`, `Wicked Rebel EX+`, ...).\nAnd use another integer variable n<sub>i</sub> to represent the number of battles we need to fight for the battle `i`.\n\nWe want to get exact honors with minimum number of battles (more efficient), so the corresponding integer programming problem is:\n\n<!-- Minimize\\ \\displaystyle\\sum_{i} n_i -->\n![formula](https://render.githubusercontent.com/render/math?math=Minimize%5C%20%5Cdisplaystyle%5Csum_%7Bi%7D%20n_i)\n\n<!-- Subject\\ to\\ \\displaystyle\\sum_{i} h_i\\times n_i -->\n![formula](https://render.githubusercontent.com/render/math?math=Subject%5C%20to%5C%20%5Cdisplaystyle%5Csum_%7Bi%7D%20h_i%5Ctimes%20n_i%20%3D%20expected%5C_honor)\n\nAdditionally, we can add additional constriants to the integer variable n<sub>i</sub> to limit the maximum number of each battle. e.g.,\n\n![formula](https://render.githubusercontent.com/render/math?math=0%3C%3Dn_i%3C%3D10)\n\n\n## How to develop\n### Setup\nUse `poetry` to setup dev environment.\n```sh\n$ poetry install\n$ poetry shell\n```\n\nUse `pre-commit` hook to check coding style.\n```sh\n$ pre-commit install -t commit-msg -t pre-commit\n```\n',
    'author': 'qq88976321',
    'author_email': 'qq88976321@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/qq88976321/gbf-beautify-honors',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
